import { createElement } from 'react';
import noop from 'lodash.noop';

// src/HTML2React/index.ts
var NON_WHITESPACE_CHARACTER = /\S/;
var HTML_SPECIAL_CHAR = /[\s>/]/;
var HTML_ATTRIBUTE_CHAR = /[=\s>/]/;
var VOID_TAGS = /* @__PURE__ */ new Set([
  "area",
  "base",
  "br",
  "col",
  "embed",
  "hr",
  "img",
  "input",
  "link",
  "meta",
  "param",
  "source",
  "track",
  "wbr"
]);
var _isVoidTag = VOID_TAGS.has.bind(VOID_TAGS);
var _handleIndex = (index) => {
  if (index < 0) {
    throw new Error("invalid html");
  }
  return index;
};
var createComponentGetter = (components) => {
  if (components) {
    const map = /* @__PURE__ */ new Map();
    const keys = Object.keys(components);
    for (let i = keys.length; i--; ) {
      const key = keys[i];
      map.set(key.toLowerCase(), components[key]);
    }
    return map.get.bind(map);
  }
  return noop;
};
var handleSegment = (segment, nodesChildren, parentMeta) => {
  if (segment && segment !== true || segment === 0) {
    const typeofSegment = typeof segment;
    let l = nodesChildren.length;
    if (typeofSegment == "object") {
      nodesChildren.push({ ...segment, key: l });
      if (parentMeta) {
        parentMeta.children.push({
          index: l,
          type: segment.type,
          parent: parentMeta
        });
      }
    } else {
      if (l && typeof nodesChildren[--l] != "object") {
        nodesChildren[l] += segment;
      } else {
        nodesChildren.push(
          typeofSegment != "number" ? segment : "" + segment
        );
        if (parentMeta) {
          parentMeta.children.length++;
        }
      }
    }
  }
};
var HTML2React = ({
  html,
  components,
  attributes = {},
  converters = {},
  processTextSegment,
  getComponent = noop,
  shouldBeIgnored,
  withMeta
}) => {
  let start = 0;
  let currentNodeChildren = [];
  let currentMeta = withMeta && { index: 0, children: [] };
  const _getComponent = createComponentGetter(components);
  const rootNodeChildren = currentNodeChildren;
  const tagsQueue = [];
  const metaQueue = [currentMeta];
  const nodesChildrenQueue = [rootNodeChildren];
  const substring = html.substring.bind(html);
  const _indexOf = html.indexOf.bind(html);
  const search = (index, regexp) => index + _handleIndex(substring(index).search(regexp));
  const indexOf = (item, index) => _handleIndex(_indexOf(item, index));
  const handleTextSegment = processTextSegment ? (nodesChildren, parentMeta, text) => {
    const segment = processTextSegment(text, parentMeta);
    if (Array.isArray(segment)) {
      for (let i = 0; i < segment.length; i++) {
        handleSegment(segment[i], nodesChildren, parentMeta);
      }
    } else {
      handleSegment(segment, nodesChildren, parentMeta);
    }
  } : (nodesChildren, parentMeta, text) => {
    nodesChildren.push(text);
    if (parentMeta) {
      parentMeta.children.length++;
    }
  };
  for (let index = _indexOf("<"), char, end, _next, tag, normalizedTag, attribute, value, props, component, meta, parentChildren = currentNodeChildren; index != -1; parentChildren = currentNodeChildren) {
    end = _indexOf(">", index + 1);
    if (end > 0) {
      while (true) {
        _next = _indexOf("<", index + 1);
        if (_next > 0 && _next < end) {
          index = _next;
        } else {
          break;
        }
      }
    } else {
      break;
    }
    if (start != index) {
      handleTextSegment(
        currentNodeChildren,
        currentMeta,
        substring(start, index)
      );
    }
    char = html[++index];
    if (char == "/") {
      tag = substring(index + 1, end).trim();
      for (let j = tagsQueue.length; j--; ) {
        if (tagsQueue[j] == tag) {
          metaQueue.length = nodesChildrenQueue.length = j + 1;
          tagsQueue.length = j;
          currentNodeChildren = nodesChildrenQueue[j];
          currentMeta = metaQueue[j];
          if (shouldBeIgnored && shouldBeIgnored(
            tag,
            currentNodeChildren[j = currentNodeChildren.length - 1].props
          )) {
            currentNodeChildren.length = j;
            if (currentMeta) {
              currentMeta.children.length = j;
            }
          }
          break;
        }
      }
    } else if (char != "!") {
      tag = substring(
        index,
        index = search(index + 1, HTML_SPECIAL_CHAR)
      ).trim();
      normalizedTag = tag.toLowerCase();
      props = {
        key: currentNodeChildren.length
      };
      component = _getComponent(normalizedTag) || getComponent(tag) || tag;
      if (currentMeta) {
        meta = {
          type: component,
          index: props.key,
          parent: currentMeta
        };
        currentMeta.children.push(meta);
        if (typeof component != "string") {
          props._meta = meta;
        }
      }
      index = search(index, NON_WHITESPACE_CHARACTER);
      while (end != index && html[index] != "/") {
        attribute = substring(
          index,
          index = search(index + 1, HTML_ATTRIBUTE_CHAR)
        ).trim();
        if (attribute in attributes) {
          attribute = attributes[attribute];
        }
        char = html[index];
        if (char != "=" && char != "/" && char != ">") {
          index = search(index + 1, NON_WHITESPACE_CHARACTER);
        }
        if (html[index] == "=") {
          char = html[index = search(index + 1, NON_WHITESPACE_CHARACTER)];
          const isWrapped = char == "'" || char == '"';
          const next = isWrapped ? indexOf(char, ++index) : search(index + 1, HTML_SPECIAL_CHAR);
          value = substring(index, next);
          index = search(next + isWrapped, NON_WHITESPACE_CHARACTER);
        } else {
          value = "true";
        }
        props[attribute] = attribute in converters ? converters[attribute](value, tag) : value;
      }
      if (normalizedTag != "script") {
        if (!_isVoidTag(normalizedTag)) {
          tagsQueue.push(tag);
          nodesChildrenQueue.push(currentNodeChildren = props.children = []);
          if (currentMeta) {
            meta.children = [];
            metaQueue.push(currentMeta = meta);
          }
        }
      } else {
        const str = substring(end + 1);
        const scriptClosingTag = /<\/\s*script\s*>/.exec(str);
        if (scriptClosingTag) {
          const indexOfNextScriptOpenTag = str.search(/<\s*script\s*>/);
          const indexOfScriptClosingTag = scriptClosingTag.index;
          if (indexOfNextScriptOpenTag == -1 || indexOfScriptClosingTag < indexOfNextScriptOpenTag) {
            const code = str.substring(0, indexOfScriptClosingTag);
            if (code) {
              props.dangerouslySetInnerHTML = { __html: code };
            }
            end += indexOfScriptClosingTag + scriptClosingTag[0].length;
          }
        }
      }
      parentChildren.push(createElement(component, props));
    } else if (html[++index] == "-" && html[++index] == "-") {
      end = indexOf("-->", index) + 2;
    }
    start = end + 1;
    index = _next;
  }
  if (start < html.length) {
    handleTextSegment(currentNodeChildren, currentMeta, substring(start));
  }
  return rootNodeChildren.length > 1 ? rootNodeChildren : rootNodeChildren[0];
};
var HTML2React_default = HTML2React;

export { HTML2React_default };
//# sourceMappingURL=chunk-RZQSRAEH.js.map
//# sourceMappingURL=chunk-RZQSRAEH.js.map